use strict;
use warnings;

use DVD::DVDProfiler;
use DVD::Netflix;

use CGI qw(:standard :html3 *table);
use Data::Dump qw(dump);
use File::Basename;
use File::Compare;
use File::Copy;
use File::Path;
use File::Spec::Functions;
use File::Temp qw(tempfile);
use Getopt::Std;
use POSIX qw(ceil);
use Storable qw(nstore retrieve);
use Text::Wrap;
use Time::localtime;
use Win32::Process;
use XML::Simple;
#perl2exe_include DVD::DVDSite
#perl2exe_include Image::Magick
#perl2exe_include Crypt::SSLeay

BEGIN
{
   unless (eval "require Image::Magick")
   {
      die 'This script requires PerlMagick (http://www.imagemagick.org/www/perl.html) to be installed';
   }
   unless (eval "require Crypt::SSLeay")
   {
      warn 'Without the Crypt::SSLeay package installed, you will not be able to log into NetFlix';
   }
}

use constant
{
   prog_version      => 1,
};

my $defconfig = {
                titletag             => 'DVD List',
                outputname           => 'DVDInfo',
                picmainname          => 'DVD Pictures',
                picbackname          => 'DVD Backup',
                dvdprofileruserid    => '',
                dvdprofilerpassword  => '',
                netflixuserid        => '',
                netflixpassword      => '',
                cachedir             => '.\dvdsummarycache',
                outputdir            => '.\dvdsummaryoutput',
                picutildir           => '.',
                mktlinfodir          => '.',
                doimages             => 1,
                palminstall          => '\program files\palm\instapp.exe',
                palminstallfiledel   => '',

                picnoimage           => 'NoImage.BMP',
                picpresscover1       => 'PressCover1.BMP',
                picpresscover2       => 'PressCover2.BMP',
                picutilcmdswitch     => 'c',
                cachedir_tempsubdir  => 'temp',
                image_format         => 'bmp',
                statename            => 'DVDSummaryState.sav',
                image_width          => 100,
                image_height         => 100,
                border_to_fit        => 1,
                main_images          => [ qw(out queued) ],
                backup_images        => [ qw(owned wishlist) ],
                config_version       => prog_version,
             };

my @configprompts = (
                outputdir            => { prompt => 'Specify a directory where you want output Palm database files (PDB\'s) to be sent to.  This directory will be created automatically if needed',
                                          dir    => 'output' },
                titletag             => 'Enter the title of the output TealInfo database (different from the file name)',
                outputname           => 'Enter the filename of the TealInfo database (do not specify the PDB, it will be appended automatically, and do not specify a directory)',
                picmainname          => 'Enter the filename of the primary TealPaint picture database (do not specify the PDB, it will be appended automatically, and do not specify a directory)',
                picbackname          => 'Enter the filename of the secondary TealPaint picture database (do not specify the PDB, it will be appended automatically, and do not specify a directory)',
                dvdprofileruserid    => 'Enter your DVD Profiler alias for the My Profiler online service (leave blank if you do not use DVD Profiler)',
                dvdprofilerpassword  => 'Enter your DVD Profiler password for the My Profiler online service; if you have made your collection public access, you can leave this blank (WARNING: This password is stored in clear text in the .xml file for this script)',
                netflixuserid        => 'Enter the email address you use with NetFlix (leave blank if you do not use NetFlix)',
                netflixpassword      => 'Enter the password you use with NetFlix (leave blank if you do not use NetFlix) (WARNING: This password is stored in clear text in the .xml file for this script)',
                cachedir             => { prompt => 'Specify a directory to store temporary files (bitmaps, etc) and "state information" from run to run of this script.  This directory will be created automatically if needed',
                                          dir    => 'output' },
                picutildir           => { prompt => 'Specify the directory where your TealPaint PICUTIL.EXE file is stored',
                                          dir    => 'mustexist',
                                          file   => 'PICUTIL.EXE' },
                mktlinfodir          => { prompt => 'Specify the directory where your TealInfo MKTLINFO.EXE file is stored',
                                          dir    => 'mustexist',
                                          file   => 'MKTLINFO.EXE' },
                doimages             => 'Specify a 0 here if you have no interest at all in cover images, to eliminate all image processing, otherwise specify 1',
                palminstall          => { prompt => 'If you want to automatically set created Palm databases for installation next time you synchronize, enter the command line of a Palm install program here.  Enter a period or non-existent file to disable automatic Palm installation (you will need to manually install files created in your output directory)',
                                          dir    => 'mustexist' },
                palminstallfiledel   => { prompt => 'If you use the default Palm install tool (INSTAPP), and try to repeatedly install the same file w/o synchronizing in between, the second install will fail because the file to be installed is already there.  Specify a directory to delete any like named file from before running the install tool.  Leave blank to not delete any files prior to running install tool',
                                          dir    => 'mustexist',
                                          canbeblank => 'yes' },
             );


my ($snam, $sdir, $sext) = File::Basename::fileparse($0, '\.[^\.]+');
my $configfilename = catfile($sdir, $snam . '.xml');
my $stdoutname = catfile($sdir, $snam . '.out');
my $stderrname = catfile($sdir, $snam . '.err');
my ($config, %opts, $ProcessObj);
eval { $config = XMLin($configfilename, suppressempty => '' ) };
getopts('ors', \%opts);

if (!defined($config) || !exists($config->{config_version}) || $config->{config_version} < prog_version || defined($opts{o}))
{
   $config = $defconfig if (!defined($config) || !exists($config->{config_version}));

   print wrap("", "", "The program needs to set up it's configuration information.  Please answer the prompts below to configure the program for running.  Pressing ENTER or entering blanks for any prompt will accept the default value.  This info will be saved and used in future runs of the program\n");
   for (my $i=0; $i<@configprompts; $i += 2)
   {
      configure($configprompts[$i], 1);
   }
}
foreach (keys(%{$defconfig}))
{
   configure($_);
}
XMLout($config, outputfile => $configfilename, noattr => 1);

if (defined($opts{s}))
{
   open(STDOUT, ">$stdoutname");
   open(STDERR, ">$stderrname");
}

print "Running build process.  To set up options, re-run with -o option.\n";

mkpath($config->{cachedir})  unless -e $config->{cachedir};
mkpath($config->{outputdir}) unless -e $config->{outputdir};

my $sitenf = DVD::Netflix->new;
my $sitedp = DVD::DVDProfiler->new;

my $stateref;
if (-e catfile($config->{cachedir}, $config->{statename}) && !defined($opts{r}))
{
   $stateref = retrieve(catfile($config->{cachedir}, $config->{statename}));
}
else
{
   $stateref = {};
}

if (exists($config->{dvdprofileruserid}) && length($config->{dvdprofileruserid}))
{
   print "Logging in to DVD Profiler...\n";
   $sitedp->login($config->{dvdprofileruserid}, $config->{dvdprofilerpassword});
   print "Getting DVD list from DVD Profiler...\n";
   $sitedp->getlist;
}

if (exists($config->{netflixuserid}) && length($config->{netflixuserid}))
{
   print "Logging in to Netflix...\n";
   $sitenf->login($config->{netflixuserid}, $config->{netflixpassword});
   print "Getting DVD list from Netflix...\n";
   $sitenf->getlist;
}

foreach my $site (($sitedp, $sitenf))
{
   foreach (values(%{$site->list}))
   {
      if ((exists($stateref->{$site->shortname}) && exists($stateref->{$site->shortname}->{$_->id})) && !defined($opts{r}))
      {
         $_->amplifydetails($stateref->{$site->shortname}->{$_->id});
      }
      else
      {
         print "Getting details for " . $_->name . "\n";
         $_->getdetails;
      }
      if ($config->{doimages})
      {
         if (defined($opts{r}))
         {
            unlink($_->fullimgname($config->{cachedir})) unless (! -e $_->fullimgname($config->{cachedir}));
            $_->getimg($config->{cachedir});
         }
         else
         {
            $_->mirrorimg($config->{cachedir}) unless (-e $_->fullimgname($config->{cachedir}));
         }
      }
   }
}

my (@catlist, @mainimglist, @backimglist);
my ($num_main_images, $num_backup_images, $num_dvds_byname, $num_dvds_bynumber) = 0;

$catlist[0] = genlist($sitedp, "owned", "colnumber");
$catlist[1] = genlist($sitedp, "wishlist", "name");
$catlist[2] = genlist($sitenf, "out", "age");
$catlist[3] = genlist($sitenf, "queued", "queue_number");

if ($config->{doimages})
{
   buildimgPDB($stateref->{'mainimglist'}, catfile($config->{cachedir}, $config->{picmainname} . ".PDB"), catfile($config->{outputdir}, $config->{picmainname} . ".PDB"), $config->{picpresscover2}, @mainimglist);
   buildimgPDB($stateref->{'backimglist'}, catfile($config->{cachedir}, $config->{picbackname} . ".PDB"), catfile($config->{outputdir}, $config->{picbackname} . ".PDB"), $config->{picpresscover1}, @backimglist);
}

my $dplist = $sitedp->detachlist;
my $nflist = $sitenf->detachlist;
nstore { $sitedp->shortname => $dplist, $sitenf->shortname => $nflist, 'mainimglist' => \@mainimglist, 'backimglist' => \@backimglist}, catfile($config->{cachedir}, $config->{statename});
$sitenf->attachlist($nflist);
$sitedp->attachlist($dplist);

open (TXTFILE, ">" . catfile($config->{cachedir}, $config->{outputname} . ".txt")) or die "Could not create " . catfile($config->{cachedir}, $config->{outputname} . ".txt") . "\n";
my $oldhandle = select (TXTFILE);

print "OUTNAME " . outquote($config->{titletag}) . "\n\n";

foreach (@catlist)
{
   foreach (@{$_})
   {
      print "TRES    dvd" . $_->attribute("sort_numeric") . "a\n";
      print "        TEXT \"<\$FONT=1><\$UNDERLINED>" . outquote($_->name, 1) . "</\$UNDERLINED></\$FONT>\"\n";
      addattribute($_, "rating");
      addattribute($_, "rating_reason", "Reason:");
      addattribute($_, "length");
      addattribute($_, "format");
      addattribute($_, "production_year", "Year:");
      addattribute($_, "genres");
      addattribute($_, "audio_tracks", "Audio:");
      addattribute($_, "director");
      addattribute($_, "description");
      addattribute($_, "starring");
   }
}

print <<TextBlock1;

LIST    aidxcatbase
        PAGES   1 2
        X       500
        Y       500
        W       10
        H       10
        TEXT
TextBlock1

foreach (@catlist)
{
   my @dvdarray = sort { $a->attribute("sort_name") <=> $b->attribute("sort_name") } @{$_};
   foreach (@dvdarray)
   {
      print "                " . outquote($_->attribute("sort_numeric")) . "\n";
   }
}

print <<TextBlock2;

LIST    imagemap
        PAGES   1 2
        X       500
        Y       500
        W       10
        H       10
        TEXT
TextBlock2

foreach (@catlist)
{
   foreach (@{$_})
   {
      print "                " . outquote($_->attribute("main_image_num")) . "\n";
   }
}

print <<TextBlock3;

LIST    bimagemap
        PAGES   1 2
        X       500
        Y       500
        W       10
        H       10
        TEXT
TextBlock3

foreach (@catlist)
{
   foreach (@{$_})
   {
      print "                " . outquote($_->attribute("backup_image_num")) . "\n";
   }
}

print <<TextBlock4;

POPLIST cat
        PAGES   1 2
	BX	108
	BY	15
	BW	50
	BH	15
	X	108
	Y	27
	W	50
	H	74
	STYLE	REGISTER
	TEXT	
                "Owned"
                "Wishlist"
                "Rented"
                "Queued"

LIST    idxcatbase
        PAGES   1 2
        X       500
        Y       500
        W       10
        H       10
        LINKS   cat
TextBlock4

my $catsize = 0;
foreach (@catlist)
{
   print "        TEXT    " . outquote($catsize) . "\n";
   $catsize += @{$_};
}

print <<TextBlock5;

LIST select
        PAGES   1
	X  2
	Y  30
	W  156
	H  112
	TABS 18
	FONT  0
        LINKS cat
	STYLE STROKE_FIND BUTTON_SELECT VERT_RULE ALIGN_RIGHT_START REGISTER
	TEXT
TextBlock5

foreach (@catlist)
{
   print "        TEXT\n";
   if (@{$_})
   {
      foreach (@{$_})
      {
         print "                " . outquote($_->attribute("sort_tag")) . " " . outquote($_->name) . "\n";
      }
   }
   else
   {
      print "                " . outquote(0) . " " . outquote('None') . "\n";
   }
}

print <<TextBlock6;

LIST aselect
        PAGES   2
	X  2
	Y  30
	W  156
	H  112
	TABS 130
	FONT  0
        LINKS cat
	STYLE STROKE_FIND BUTTON_SELECT VERT_RULE ALIGN_RIGHT_END REGISTER
TextBlock6

foreach (@catlist)
{
   print "        TEXT\n";
   if (@{$_})
   {
      my @dvdarray = sort { $a->attribute("sort_name") <=> $b->attribute("sort_name") } @{$_};
      foreach (@dvdarray)
      {
         print "                " . outquote($_->name) . " " . outquote($_->attribute("sort_tag")) . "\n";
      }
   }
   else
   {
      print "                " . outquote(0) . " " . outquote('None') . "\n";
   }
}

print <<TextBlock7;

MATH    mthcatbase
        PAGES   1 2
        X       500
        Y       500
        W       10
        H       10
        TEXT    "\@idxcatbase #select +"

MATH    bmthcatbase
        PAGES   1 2
        X       500
        Y       500
        W       10
        H       10
        TEXT    "\@idxcatbase #aselect +"

MATH    amthcatbase
        PAGES   1 2
        X       500
        Y       500
        W       10
        H       10
        TEXT    "\@aidxcatbase[\@bmthcatbase]"

MATH    imagesel
        PAGES   1 2
        X       500
        Y       500
        W       10
        H       10
        TEXT    "\@imagemap[\@mthcatbase]"

MATH    aimagesel
        PAGES   1 2
        X       500
        Y       500
        W       10
        H       10
        TEXT    "\@imagemap[\@amthcatbase]"

MATH    bimagesel
        PAGES   1 2
        X       500
        Y       500
        W       10
        H       10
        TEXT    "\@bimagemap[\@mthcatbase]"

MATH    baimagesel
        PAGES   1 2
        X       500
        Y       500
        W       10
        H       10
        TEXT    "\@bimagemap[\@amthcatbase]"

POPWINDOW  details
        PAGES   1
	BX	2
	BY	145
	BW	35
	BH	14
        X       0
        Y       0
        W       160
        H       160
	BTEXT	"Details"
        LINKS  mthcatbase
TextBlock7

foreach (@catlist)
{
   foreach (@{$_})
   {
      print '        TEXT @dvd' . $_->attribute("sort_numeric") . "a\n";
   }
}

print <<TextBlock8;

POPWINDOW  adetails
        PAGES   2
	BX	2
	BY	145
	BW	35
	BH	14
        X       0
        Y       0
        W       160
        H       160
	BTEXT	"Details"
        LINKS  amthcatbase
TextBlock8

foreach (@catlist)
{
   foreach (@{$_})
   {
      print '        TEXT @dvd' . $_->attribute("sort_numeric") . "a\n";
   }
}

print <<TextBlock9;

SETPAGE
        PAGES   1
        BX      2
        BY      15
        BW      60
        BH      12
        TARGET  "2"
        BTEXT   "Sort Name"

SETPAGE
        PAGES   2
        BX      2
        BY      15
        BW      60
        BH      12
        TARGET  "1"
        BTEXT   "Sort Number"

POPFIND
        PAGES   1
	BX	128
	BY	145
	BW	30
	BH	14
        BTEXT   "Find"
        TARGET  "\@select"

POPFIND
        PAGES   2
	BX	128
	BY	145
	BW	30
	BH	14
        BTEXT   "Find"
        TARGET  "\@aselect"

POPIMAGE
        PAGES   1
	BX	40
	BY	145
	BW	41
	BH	14
        X       30
        Y       30
        W       100
        H       100
        SX      0
        SY      0
	BTEXT	"Cover1"
        LINKS   imagesel
TextBlock9

print "        DATABASE " . outquote($config->{picmainname}) . "\n";

print <<TextBlock10;
        RECORD  0

POPIMAGE
        PAGES   2
	BX	40
	BY	145
	BW	41
	BH	14
        X       30
        Y       30
        W       100
        H       100
        SX      0
        SY      0
	BTEXT	"Cover1"
        LINKS   aimagesel
TextBlock10

print "        DATABASE " . outquote($config->{picmainname}) . "\n";

print <<TextBlock11;
        RECORD  0

POPIMAGE
        PAGES   1
	BX      84
	BY      145
	BW      41
	BH      14
        X       30
        Y       30
        W       100
        H       100
        SX      0
        SY      0
	BTEXT   "Cover2"
        LINKS   bimagesel
TextBlock11

print "        DATABASE " . outquote($config->{picbackname}) . "\n";

print <<TextBlock12;
        RECORD  0

POPIMAGE
        PAGES   2
	BX      84
	BY      145
	BW      41
	BH      14
        X       30
        Y       30
        W       100
        H       100
        SX      0
        SY      0
	BTEXT   "Cover2"
        LINKS   baimagesel
TextBlock12

print "        DATABASE " . outquote($config->{picbackname}) . "\n";

print <<TextBlock13;
        RECORD  0

POPWINDOW
        PAGES   1 2
	BX	127
	BY	1
	BW	15
	BH	11
	BTEXT	"?"
	X	5
	Y	5
	W	150
	H	150
	TEXT
		"DVD Summary"
		" "
"This folio shows information about movies in your collection, that you have rented from " \
"Netflix, that you wish for, or that are queued by Netflix to send to you. "

LABEL
        PAGES   1 2
	X	1
	Y	1
	FONT	1
	STYLE	INVERTED	
	TEXT	
	"                                                               "

LABEL
        PAGES   1 2
	X	5
	Y	1
	FONT	1
	STYLE	INVERTED	
TextBlock13

print "        TEXT    " . outquote($config->{titletag}) . "\n";

print <<TextBlock14;

YEAR    myyear

MONTH   mymonth

DAY     myday

LIST    monthdays
        PAGES   1 2
        X       500
        Y       500
        W       10
        H       10
        TEXT
                "0"
                "31"
                "59"
                "90"
                "120"
                "151"
                "181"
                "212"
                "243"
                "273"
                "304"
                "334"

LABEL
        PAGES   1 2
        X       68
        Y       16
        FONT    0
        STYLE
        TEXT    "Age:+"

MATH
        PAGES   1 2
        X       91
        Y       16
        W       16
        H       12
        STYLE   NO_BORDER
TextBlock14

my $tm = localtime;
my @monthdays = qw(0 31 59 90 120 151 181 212 243 273 304 334);

print '        TEXT    "#myyear 365 * @monthdays[#mymonth] + #myday + ' . (($tm->year%100)*365 + $monthdays[$tm->mon] + $tm->mday-1) . " -\"\n";

select($oldhandle);
close(TXTFILE);

unlink(catfile($config->{outputdir}, $config->{outputname} . ".PDB")) if (-e catfile($config->{outputdir}, $config->{outputname} . ".PDB"));
if (!(-e catfile($config->{cachedir}, $config->{outputname} . ".old.txt")) || compare(catfile($config->{cachedir}, $config->{outputname} . ".txt"), catfile($config->{cachedir}, $config->{outputname} . ".old.txt")) || defined($opts{r}))
{
   unlink(catfile($config->{cachedir}, $config->{outputname} . ".PDB")) if (-e catfile($config->{cachedir}, $config->{outputname} . ".PDB"));
   system(catfile($config->{mktlinfodir}, "mktlinfo.exe"), catfile($config->{cachedir}, $config->{outputname} . ".txt"), catfile($config->{cachedir}, $config->{outputname} . ".PDB"), $config->{titletag});
   unlink(catfile($config->{cachedir}, $config->{outputname} . ".old.txt")) if (-e catfile($config->{cachedir}, $config->{outputname} . ".old.txt"));
   move(catfile($config->{cachedir}, $config->{outputname} . ".txt"), catfile($config->{cachedir}, $config->{outputname} . ".old.txt"));
   move(catfile($config->{cachedir}, $config->{outputname} . ".PDB"), catfile($config->{outputdir}, $config->{outputname} . ".PDB"));
#   if (defined($ProcessObj))
#   {
#      my $exitcode=0;
#      $ProcessObj->Kill($exitcode);
#      undef $ProcessObj;
#   }
#   Win32::Process::Create(my $ProcessObj, $config->{palminstall}, 'instapp "' . File::Spec->rel2abs(catfile($config->{outputdir}, $config->{outputname} . '.PDB"')), 0, NORMAL_PRIORITY_CLASS, File::Spec->curdir());
   installtopalm(File::Spec->rel2abs(catfile($config->{outputdir}, $config->{outputname} . '.PDB')));
}

print "Processing complete, see " . $config->{outputdir} . " for any files to sync to your Palm device.\n";

sub genlist
{
   my ($siteref, $typematch, $sortkey) = @_;

   my @dvdarray = ();
   foreach (values(%{$siteref->list}))
   {
      push(@dvdarray, $_) if ($_->type eq $typematch);
   }

   @dvdarray = sort { $a->name cmp $b->name } @dvdarray;

   foreach (@dvdarray)
   {
      $_->attribute("sort_name", $num_dvds_byname++);
   }

   @dvdarray = sort {
                  return ($a->attribute($sortkey) <=> $b->attribute($sortkey)) if (($a->attribute($sortkey) =~ /^\d+$/) && ($b->attribute($sortkey) =~ /^\d+$/));
                  return ($a->attribute($sortkey) cmp $b->attribute($sortkey));
               } @dvdarray;

   foreach (@dvdarray)
   {
      $_->attribute("sort_numeric", $num_dvds_bynumber++);
      if ($config->{doimages} && -e $_->fullimgname($config->{cachedir}))
      {
         if (! -e $_->fullimgname($config->{cachedir}, $config->{image_format}, $config->{cachedir_tempsubdir}))
         {
            my $image = Image::Magick->new;
            my $imagesread = $image->Read($_->fullimgname($config->{cachedir}));
            if ($imagesread == 1)
            {
               my ($width, $height) = $image->get('width', 'height');
               if ($width != $config->{image_width} || $height != $config->{image_height})
               {
                  my $adjfactor = $config->{image_width} / $width;
                  $adjfactor = $config->{image_height} / $height if (($config->{image_height} / $height) < $adjfactor);
                  $image->Resize(width=>int($adjfactor*$width), height=>int($adjfactor*$height));
               }
               ($width, $height) = $image->get('width', 'height');
               if (($width != $config->{image_width} || $height != $config->{image_height}) && $config->{border_to_fit})
               {
                  $image->Border(width=>ceil(($config->{image_width}-$width)/2), height=>0) if (ceil(($config->{image_width}-$width)/2));
                  $image->Border(height=>ceil(($config->{image_height}-$height)/2), width=>0) if ceil(($config->{image_height}-$height)/2);
                  $image->Crop(width=>$config->{image_width}, height=>$config->{image_height});
               }
               $image->Quantize(colors=>255);
               $image->Set(magick=>$config->{image_format}, compression=>'None');
               $image->Write(filename=>$_->fullimgname($config->{cachedir}, $config->{image_format}, $config->{cachedir_tempsubdir}));
            }
         }
         if (grep(/^$typematch$/, @{$config->{main_images}}))
         {
            $_->attribute("main_image_num",   $num_main_images++ + 2);
            $_->attribute("backup_image_num", 1);
            push @mainimglist, $_->fullimgname($config->{cachedir}, $config->{image_format}, $config->{cachedir_tempsubdir});
         }
         elsif (grep(/^$typematch$/, @{$config->{backup_images}}))
         {
            $_->attribute("main_image_num",   1);
            $_->attribute("backup_image_num", $num_backup_images++ + 2);
            push @backimglist, $_->fullimgname($config->{cachedir}, $config->{image_format}, $config->{cachedir_tempsubdir});
         }
         else
         {
            $_->attribute("main_image_num",   0);
            $_->attribute("backup_image_num", 0);
         }
      }
      else
      {
         $_->attribute("main_image_num",   0);
         $_->attribute("backup_image_num", 0);
      }
   }

   my $i=1;
   foreach (@dvdarray)
   {
      if ($sortkey eq "name")
      {
         $_->attribute("sort_tag", $i++);
      }
      else
      {
         $_->attribute("sort_tag", substr($_->attribute($sortkey), 0, 3));
      }
   }

   return \@dvdarray;
}

sub addattribute
{
   my ($dvd, $attr, $label) = @_;

   $label = ucfirst($attr) . ":" if (!defined($label));
   my $attrval = $dvd->attribute($attr);
   print "             \"<\$FONT=1>$label</\$FONT> " . outquote($attrval, 1) . "\"\n" if ($dvd->present($attr));
}

sub outquote
{
   my ($str, $noquote) = @_;

   $str =~ s/\"/\"\"/g;
   if (defined($noquote))
   {
      return $str;
   }
   else
   {
      return '"' . $str . '"';
   }
}

sub buildimgPDB
{
   my ($stateimglist, $pdbfile, $outfile, $coverpic, @imglist) = @_;

   unlink($outfile) if (-e $outfile);
   my $i=0;
   if (defined($stateimglist) && !defined($opts{r}))
   {
      if (@imglist == @{$stateimglist})
      {
         for ($i=0; $i < @imglist; $i++)
         {
            last if ($imglist[$i] ne $stateimglist->[$i]);
         }
      }
   }
   if ($i < @imglist)
   {
      unlink($pdbfile) if (-e $pdbfile);
      system(catfile($config->{picutildir}, "picutil.exe"), "-n", $pdbfile);
      system(catfile($config->{picutildir}, "picutil.exe"), "-" . $config->{picutilcmdswitch}, $pdbfile, $config->{picnoimage});
      system(catfile($config->{picutildir}, "picutil.exe"), "-" . $config->{picutilcmdswitch}, $pdbfile, $coverpic);
      foreach (@imglist)
      {
         system(catfile($config->{picutildir}, "picutil.exe"), "-" . $config->{picutilcmdswitch}, $pdbfile, $_);
      }
      move($pdbfile, $outfile);
#      if (defined($ProcessObj))
#      {
#         my $exitcode=0;
#         $ProcessObj->Kill($exitcode);
#         undef $ProcessObj;
#      }
#      Win32::Process::Create($ProcessObj, $config->{palminstall}, 'instapp "' . File::Spec->rel2abs($outfile) . '"', 0, NORMAL_PRIORITY_CLASS, File::Spec->curdir());
      installtopalm(File::Spec->rel2abs($outfile));
      $pdbfile =~ s/\.PDB$/\.\$\$\$/;
      unlink($pdbfile) if (-e $pdbfile);
   }
}

sub trim
{
   my @out = @_;

   for (@out)
   {
      s/^\s+//;
      s/\s+$//;
   }
   return wantarray ? @out : $out[0];
}

sub installtopalm
{
   my ($pdbfile) = @_;

   if (defined($ProcessObj))
   {
      my $exitcode=0;
      $ProcessObj->Kill($exitcode);
      undef $ProcessObj;
   }
   my ($vol, $path, $file) = File::Spec->splitpath($pdbfile);
   if ($config->{palminstallfiledel} ne '')
   {
      my ($vol, $pth, $fil) = File::Spec->splitpath($pdbfile);
      unlink(catfile($config->{palminstallfiledel}, $fil)) if (-e catfile(catfile($config->{palminstallfiledel}, $fil)));
   }
   Win32::Process::Create($ProcessObj, $config->{palminstall}, 'instapp "' . $pdbfile . '"', 0, NORMAL_PRIORITY_CLASS, File::Spec->curdir());
}

sub configure
{
   my ($configkey, $force) = @_;
   my ($resp, $prompt, $i, $configifo);

   return unless (!exists($config->{$configkey}) || (defined($force) && $force));
   $config->{$configkey} = $defconfig->{$configkey} unless exists($config->{$configkey});
   for ($i=0; $i<@configprompts; $i += 2)
   {
      last if ($configprompts[$i] eq $configkey);
   }
   return if ($i >= @configprompts);
   $configifo = $configprompts[$i+1];
   if (ref($configifo))
   {
      $prompt = $configifo->{prompt};
   }
   else
   {
      $prompt = $configifo;
   }
   do
   {
      print "\n" . wrap("==> ", "    ", $prompt . " [DEFAULT=" . $config->{$configkey} . "]: ");
      chomp($resp = <STDIN>);
      $config->{$configkey} = trim($resp) if (length(trim($resp)));
      if (ref($configifo) && exists($configifo->{dir}))
      {
         unless (exists($configifo->{canbeblank}) && $configifo->{canbeblank} eq 'yes' && $config->{$configkey} eq '')
         {
            if (!File::Spec->file_name_is_absolute($config->{$configkey}))
            {
               print wrap("==> ", "    ", "The directory you specified is a relative directory, and will vary as the current directory varies when this program is run.  Do you want to make this an absolute directory (" . File::Spec->rel2abs($config->{$configkey}) . ")?  Press ENTER on a blank line to make it absolute, or press r to leave it as the original relative value entered: ");
               chomp($resp = <STDIN>);
               $config->{$configkey} = File::Spec->rel2abs($config->{$configkey}) if (trim($resp) eq '');
            }
            if ($configifo->{dir} eq 'mustexist')
            {
               my $fil = $config->{$configkey};
               $fil = catfile($fil, $configifo->{file}) if (exists($configifo->{file}));
               if (! -e $fil)
               {
                  print wrap("==> ", "    ", "The file you referenced (" . $fil . "} does not exist.  That function will not work properly unless it exists when the script runs.  Enter any non-blank value to be able to re-enter your answer: ");
                  chomp($resp = <STDIN>);
                  redo if (trim($resp) ne '');
               }
            }
         }
      }
   }
   until 1;
}
